/*
 * Copyright:
 * ----------------------------------------------------------------
 * This confidential and proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 *   (C) COPYRIGHT 2013 ARM Limited
 *       ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 * ----------------------------------------------------------------
 * File:     aptsc.c
 * Release:  Version 2.0
 * ----------------------------------------------------------------
 */

/*
 * Code implementation file for the Touch Screen Controller interface.
 */

#include <stdio.h>
#include <stdlib.h>

#include "GLCD_SPI_MPS2.h"
#include "TSC_I2C_MPS2.h"

#include "../main/common.h"
#include "aptsc.h"
#include "icons.h"

// Initialise TSC
int apTSC_INIT(void)
{
	unsigned int din, err;

	err = FALSE;

	// Clear the TSC I2C interface as there is no RESET control
    TSC_I2C_clear();

    // Read and check the I2C chip ID
    din = TSC_I2C_read(TSC_I2C_CRID, TSC_I2C_ADDR, 2);
    if (din != TSC_I2C_CID)
    {
        apDebug("ERROR: TSC ID:0x%04X\n", din);
        err = TRUE;
    }
    else
    {
        apDebug("TSC ID:0x%04X\n", din);

		// Initialise the TSC I2C interface
		TSC_I2C_write(0x40, 0x03, TSC_I2C_ADDR);

		// Reset Touch-screen controller
		TSC_I2C_write(0x03, 0x03, TSC_I2C_ADDR);
		apSleepus(10000);

		// Enable TSC and ADC
		TSC_I2C_write(0x04, 0x0C, TSC_I2C_ADDR);

		// Enable Touch detect, FIFO
		TSC_I2C_write(0x0A, 0x01, TSC_I2C_ADDR);

		// Set sample time , 12-bit mode
		TSC_I2C_write(0x20, 0x39, TSC_I2C_ADDR);
		apSleepus(2000);

		// ADC frequency 3.25 MHz
		TSC_I2C_write(0x21, 0x01, TSC_I2C_ADDR);

		// Set TSC_CFG register
		TSC_I2C_write(0x41, 0xF5, TSC_I2C_ADDR);

		// Threshold for FIFO
		TSC_I2C_write(0x4A, 0x02, TSC_I2C_ADDR);

		// FIFO reset
		TSC_I2C_write(0x4B, 0x01, TSC_I2C_ADDR);
		apSleepus(1000);
		TSC_I2C_write(0x4B, 0x00, TSC_I2C_ADDR);

		// Fraction Z
		TSC_I2C_write(0x56, 0x07, TSC_I2C_ADDR);

		// Drive 50 mA typical
		TSC_I2C_write(0x58, 0x01, TSC_I2C_ADDR);

		// Enable TSC
		TSC_I2C_write(0x40, 0x01, TSC_I2C_ADDR);

		// Clear interrupt status
		TSC_I2C_write(0x0B, 0xFF, TSC_I2C_ADDR);

		// Enable global interrupt
		TSC_I2C_write(0x09, 0x01, TSC_I2C_ADDR);

		// Clear status register
		TSC_I2C_write(0x0B, 0xFF, TSC_I2C_ADDR);
    }

    return err;
}

// TSC read pen down, X,Y
unsigned int apTSC_READXY(unsigned int *PENX, unsigned int *PENY)
{
    unsigned int loop, din, isr;
    unsigned int X, Y, XPXL, YPXL, pendown;
    //unsigned int Z;

	pendown  = FALSE;
	*PENX    = 0;
	*PENY    = 0;

	// Read interrupt status
	isr = TSC_I2C_read(0x0B, TSC_I2C_ADDR, 1);

	// Test for FIFO_TH interrupt
	if (isr & 0x02)
	{
		// Empty the FIFO
		loop = TSC_I2C_read(0x4C, TSC_I2C_ADDR, 1);
		while (loop > 1)
		{
			din = TSC_I2C_read(0xD7, TSC_I2C_ADDR, 4);
			loop--;
		}

		// Clear the interrupt (must be immediately after FIFO empty)
		TSC_I2C_write(0x0B, isr, TSC_I2C_ADDR);

		// Read coordinates
		din = TSC_I2C_read(0xD7, TSC_I2C_ADDR, 4);
		X = (din >> 20) & 0x00000FFF;
		Y = (din >>  8) & 0x00000FFF;
		//Z = (din >>  0) & 0x0000000F;

		// printf("X:0x%04X, Y:0x%04X, Z:0x%02X\n", X, Y, Z);

		// Calculate the pixel position (X/Y are swapped)
		XPXL = LCD_WIDTH - ((Y * 10) / (TSC_MAXVAL / LCD_WIDTH)) + TSC_XOFF;
		YPXL = ((X * 10) / (TSC_MAXVAL / LCD_HEIGHT)) - TSC_YOFF;
		XPXL = (XPXL & 0x80000000) ? 0 : XPXL;
		YPXL = (YPXL & 0x80000000) ? 0 : YPXL;
		XPXL = (XPXL >= LCD_WIDTH ) ? LCD_WIDTH  - 1 : XPXL;
		YPXL = (YPXL >= LCD_HEIGHT) ? LCD_HEIGHT - 1 : YPXL;

		// Move to new position if pen lifted
		if (!pendown)
		{
			*PENX = XPXL;
			*PENY = YPXL;
		}
	}

	// Read interrupt status
	din = TSC_I2C_read(0x40, TSC_I2C_ADDR, 1);

	// Read the raw pen down status
	if (din & 0x80)
		pendown = TRUE;

    return pendown;
}

// Move slider horizontal
void apTSC_setslide(unsigned int xpos, unsigned int ypos, unsigned int scol, unsigned int nxpos)
{
    unsigned short change_color[2];

	// Clear the old slider
    GLCD_BoxSize(xpos - 10, xpos + 5, ypos, ypos + 23, 0xFFFF);

	// Draw the new slider
    change_color[0] = Black;
    change_color[1] = scol & 0xFFFF;
    GLCD_ChangeBitMap (nxpos - 10, nxpos + 5, ypos, ypos + 23, sliderData, 1, change_color);
}

// Move slider vertical
void apTSC_setslidev(unsigned int xpos, unsigned int ypos, unsigned int scol, unsigned int nypos)
{
    unsigned short change_color[2];

	// Clear the old slider
    GLCD_BoxSize(xpos, xpos + 23, ypos, ypos + 15, 0xFFFF);

	// Draw the new slider
    change_color[0] = Black;
    change_color[1] = scol & 0xFFFF;
    GLCD_ChangeBitMap (xpos, xpos + 23, nypos, nypos + 15, slidervData, 1, change_color);
}
